<?php

namespace EventPortal\Tests\Unit\FormProgress;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Delegate\Models\DelegateFormStep;
use EventPortal\Domain\Event\Form\Enums\State;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\FormProgress\FormProgress;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class FormProgressTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test **/
    public function can_get_form_progress()
    {
        $event = factory(Event::class)->create([
            'slug' => 'test-event',
        ]);

        $eventDelegate = factory(EventDelegate::class)
            ->state('registering')
            ->create([
                'event_id' => $event->id,
            ]);

        $form = factory(Form::class)
            ->create([
                'event_id' => $event->id,
                'name' => 'Registration',
                'slug' => 'registration',
            ]);

        $formStepOne = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 1,
            'name' => 'Step One',
        ]);
        $stepOne = factory(DelegateFormStep::class)->create([
            'form_step_id' => $form->id,
            'delegate_id' => $eventDelegate->delegate->id,
            'state' => State::Complete(),
        ]);

        $formStepTwo = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 2,
            'name' => 'Step Two',
        ]);

        $progress = FormProgress::make()
            ->forForm($form)
            ->forEventDelegate($eventDelegate)
            ->get();

        $expectedProgress = [
            [
                'value' => 1,
                'label' => 'Step One',
                'completed' => true,
            ],
            [
                'value' => 2,
                'label' => 'Step Two',
                'completed' => false,
            ],
        ];

        $expectedActions = [
            Button::make('Continue registration')
                ->method(InertiaMethod::VISIT())
                ->route('delegate.form.index', ['event' => $event->slug, 'form' => $form->slug])
                ->icon('chevron-right', 'right'),
        ];

        $this->assertSame('Registration', $progress->title);
        $this->assertEquals($expectedActions, $progress->actions);
        $this->assertSame($expectedProgress, $progress->progress);
    }

    /** @test */
    public function can_get_form_progress_as_array()
    {
        $event = factory(Event::class)->create([
            'slug' => 'test-event',
        ]);

        $eventDelegate = factory(EventDelegate::class)
            ->state('registering')
            ->create([
                'event_id' => $event->id,
            ]);

        $form = factory(Form::class)
            ->create([
                'event_id' => $event->id,
                'name' => 'Registration',
                'slug' => 'registration',
            ]);

        $formStepOne = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 1,
            'name' => 'Step One',
        ]);
        $stepOne = factory(DelegateFormStep::class)->create([
            'form_step_id' => $form->id,
            'delegate_id' => $eventDelegate->delegate->id,
            'state' => State::Complete(),
        ]);

        $formStepTwo = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 2,
            'name' => 'Step Two',
        ]);

        $progress = FormProgress::make()
            ->forForm($form)
            ->forEventDelegate($eventDelegate)
            ->get()
            ->toArray();

        $expectedProgress = [
            [
                'value' => 1,
                'label' => 'Step One',
                'completed' => true,
            ],
            [
                'value' => 2,
                'label' => 'Step Two',
                'completed' => false,
            ],
        ];

        $expectedActions = [
            Button::make('Continue registration')
                ->method(InertiaMethod::VISIT())
                ->route('delegate.form.index', ['event' => $event->slug, 'form' => $form->slug])
                ->icon('chevron-right', 'right'),
        ];

        $this->assertTrue(is_array($progress));
        $this->assertSame('Registration', $progress['title']);
        $this->assertEquals($expectedActions, $progress['actions']);
        $this->assertSame($expectedProgress, $progress['progress']);
    }
}
